##########################################################################
##
##
##	WARNING: This code is not pretty, and not general. May not be 
##	easily adapted to other applications.
##
##
##	This file replications Table 1 of the paper -- NSW Analysis 
##
##	Author: Max H. Farrell
##	Date: 2014-06-26
##
##	Description / notes: 
##	-DW = Dehejia-Wahba2002_REStat
##	.DW02 = Smith-Todd2005_JoE
##	-Use the same LaLonde sample as DW (the re74 sample)
##	-Use the PSID controls (not CPS)
##	-We do NOT follow .DW02, who combine NSW treated+controls into a new
##	T==1 group, then use the PSID as the T==0 group. The NSW controls
##	are eligible non-participants, so for prop score estimation 
##	giving them T==1 is OK.
##	-We only apply the EIF estimator, with different specifications.
##	-Directly related, we use the SAME specification for mu_t(x) as
##	for p_t(x). Prior work (DW,.DW02, etc) only talk about prop. scores
##	so we have to use the same specification. This isn't optimal, 
##	but it's the most fair.
##
##########################################################################

rm(list=ls())


library("foreign")
library("MASS")
library("glmnet")	#lasso, group lasso, and ridge, for outcome models, LPM, mlogit. Also, unpenalized mlogit via glmnet is far faster than the mlogit package.
library("grplasso")	#glmnet group lasso requires the same n per group (ie multi-task learning), which is perfect for mlogit but wrong for the outcome model.
# library("mlogit")	#slower than unpenalized estimation in glmnet, but glmnet won't fit only an intercept
library("nnet")	#quicker multinomial logit
library("gglasso")






##################################
## 			 	##
## 	Import all data 	##
## 			 	##
##################################


  import.LaLonde.data <- function(file.name=c("nsw_control"), variable.names=names74) {
	data <- read.table(paste(file.name,".txt",sep=""))
	colnames(data) <- variable.names
	return(data)
      }
  names74 <- c("treat", "age", "education", "black", "hispanic", "married", "nodegree", "re74", "re75", "re78")
    #From Dehejia's webpage:
      #treatment indicator (1 if treated, 0 if not treated), age, education, Black (1 if black, 0 otherwise), Hispanic (1 if Hispanic, 0 otherwise), married (1 if married, 0 otherwise), nodegree (1 if no degree, 0 otherwise), RE74 (earnings in 1974), RE75 (earnings in 1975), and RE78 (earnings in 1978)


  nswre74_control <- import.LaLonde.data(file.name="nswre74_control",variable.names=names74)
  nswre74_treated <- import.LaLonde.data(file.name="nswre74_treated",variable.names=names74)
  psid_control1 <- import.LaLonde.data(file.name="psid_controls",variable.names=names74)








##################################
## 			 	##
## 	Prepare PSID data 	##
## 			 	##
##################################

psid <- rbind(nswre74_treated,psid_control1)

Y <- psid[,"re78"]
T <- psid[,"treat"]
n <- length(T)
n.per.treatment <- as.vector(table(T))
Treatments <- 2





##################################
## 			 	##
## 	No Model Selection 	##
## 			 	##
##################################
  mu11.experimental <- mean(nswre74_treated[,"re78"])
  mu01.experimental <- mean(nswre74_control[,"re78"])
  tot.experimental <- mu11.experimental - mu01.experimental
  
  n.per.treatment.experimental <- c(dim(nswre74_control)[1],dim(nswre74_treated)[1])
  n.experimental <- n.per.treatment.experimental[1]+n.per.treatment.experimental[2]	##need later for standard errors
  p.1.experimental <- n.per.treatment.experimental[2]/n.experimental
  var.tot.experimental <- mean(nswre74_treated[,"re78"]^2)/p.1.experimental + mean(nswre74_control[,"re78"]^2)/(1-p.1.experimental) -  tot.experimental^2
  
  

  

##################################
## 			 	##
## 	No Model Selection 	##
## 			 	##
##################################

## Main effects and all squared terms
  X <- cbind(psid[,"age"], psid[,"education"], psid[,"married"], psid[,"black"], psid[,"hispanic"], psid[,"re74"], psid[,"re75"], psid[,"age"]^2, psid[,"education"]^2, psid[,"re74"]^2, psid[,"re75"]^2)
  X <- scale(X,center=FALSE,scale=TRUE)
  
  #Linear part
    Y.1.squared <- X%*%(solve(crossprod(X[T==1,]))%*%crossprod(X[T==1,],Y[T==1]))
    Y.0.squared <- X%*%(solve(crossprod(X[T==0,]))%*%crossprod(X[T==0,],Y[T==0]))
    
  #logit part
    p.1.squared <- multinom(T~X-1, trace=FALSE)$fitted.values
    
  #Impose common support
    indexes.to.drop <- which(p.1.squared < min(p.1.squared[T==1]) | max(p.1.squared[T==1]) < p.1.squared)
    if (length(indexes.to.drop)==0) {indexes.to.drop <- n+1}	#R throws a wobbly if [-indexes.to.drop] is negating an empty set. 
    n.per.treatment.squared <- as.vector(table(T[-indexes.to.drop]))
    n.trim <- n.per.treatment.squared[1]+n.per.treatment.squared[2]
    
  #Treatment Effects on the Treated
    p.1 <- n.per.treatment.squared[2]/n.trim
    mu01.eif.squared <- mean((T[-indexes.to.drop]==1)*Y.0.squared[-indexes.to.drop]) + mean((T[-indexes.to.drop]==0)*p.1.squared[-indexes.to.drop]*(Y[-indexes.to.drop] - Y.0.squared[-indexes.to.drop])/(1-p.1.squared[-indexes.to.drop]))
    mu01.eif.squared <- mu01.eif.squared/p.1
    tot.eif.squared <- mu11.experimental - mu01.eif.squared
    
    var.tot.eif.squared <- mean( (T[-indexes.to.drop]==1)*(Y[-indexes.to.drop] - Y.0.squared[-indexes.to.drop] - tot.eif.squared)^2 )  + mean( (T[-indexes.to.drop]==0)*((Y[-indexes.to.drop] - Y.0.squared[-indexes.to.drop])^2)*(p.1.squared[-indexes.to.drop]^2/(1-p.1.squared[-indexes.to.drop])^2) )
    var.tot.eif.squared <- var.tot.eif.squared/p.1^2



 
## Dehejia-Wahba2002_REStat Table 3 Specification
## Smith-Todd2005_JoE Table 3 Specification
      
  X <- cbind(1,psid[,"age"], psid[,"age"]^2, psid[,"education"], psid[,"education"]^2, psid[,"married"], psid[,"nodegree"], psid[,"black"], psid[,"hispanic"], psid[,"re74"], psid[,"re74"]^2, psid[,"re75"], psid[,"re75"]^2, psid[,"re74"]==0, psid[,"re75"]==0, (psid[,"re74"]==0)*(psid[,"hispanic"]))
  #For later saving
    X.DW02 <- X
  X <- scale(X,center=FALSE,scale=TRUE)
  
  #Linear part
    Y.1.DW02 <- X%*%(solve(crossprod(X[T==1,]))%*%crossprod(X[T==1,],Y[T==1]))
    Y.0.DW02 <- X%*%(solve(crossprod(X[T==0,]))%*%crossprod(X[T==0,],Y[T==0]))
    
  #logit part
    p.1.DW02 <- multinom(T~X-1, trace=FALSE)$fitted.values
    
  #Impose common support
    indexes.to.drop <- which(p.1.DW02 < min(p.1.DW02[T==1]) | max(p.1.DW02[T==1]) < p.1.DW02)
    if (length(indexes.to.drop)==0) {indexes.to.drop <- n+1}	#R throws a wobbly if [-indexes.to.drop] is negating an empty set. 
    n.per.treatment.DW02 <- as.vector(table(T[-indexes.to.drop]))
    n.trim <- n.per.treatment.DW02[1]+n.per.treatment.DW02[2]
    
  #Treatment Effects on the Treated
    p.1 <- n.per.treatment.DW02[2]/n.trim
    mu01.eif.DW02 <- mean((T[-indexes.to.drop]==1)*Y.0.DW02[-indexes.to.drop]) + mean((T[-indexes.to.drop]==0)*p.1.DW02[-indexes.to.drop]*(Y[-indexes.to.drop] - Y.0.DW02[-indexes.to.drop])/(1-p.1.DW02[-indexes.to.drop]))
    mu01.eif.DW02 <- mu01.eif.DW02/p.1
    tot.eif.DW02 <- mu11.experimental - mu01.eif.DW02
    
    var.tot.eif.DW02 <- mean( (T[-indexes.to.drop]==1)*(Y[-indexes.to.drop] - Y.0.DW02[-indexes.to.drop] - tot.eif.DW02)^2 )  + mean( (T[-indexes.to.drop]==0)*((Y[-indexes.to.drop] - Y.0.DW02[-indexes.to.drop])^2)*(p.1.DW02[-indexes.to.drop]^2/(1-p.1.DW02[-indexes.to.drop])^2) )
    var.tot.eif.DW02 <- var.tot.eif.DW02/p.1^2

    
    
   
    
    
    
    
  

##################################
## 			 	##
## 	Model Selection	 	##
## 			 	##
##################################




  
#### 2. X = Many more terms for flexibility	####

  X.main <- cbind(1, psid[,"age"], psid[,"education"], psid[,"married"], psid[,"black"], psid[,"hispanic"], psid[,"re74"], psid[,"re75"], psid[,"nodegree"], psid[,"re74"]==0, psid[,"re75"]==0)

  X.cont.interactions <- cbind(
    psid[,"age"]*psid[,"married"],
    psid[,"age"]*psid[,"nodegree"],
    psid[,"age"]*psid[,"black"],
    psid[,"age"]*psid[,"hispanic"],
    psid[,"age"]*(psid[,"re74"]==0),
    psid[,"age"]*(psid[,"re75"]==0),
    psid[,"education"]*psid[,"married"],
    psid[,"education"]*psid[,"nodegree"],
    psid[,"education"]*psid[,"black"],
    psid[,"education"]*psid[,"hispanic"],
    psid[,"education"]*(psid[,"re74"]==0),
    psid[,"education"]*(psid[,"re75"]==0),
    psid[,"re74"]*psid[,"married"],
    psid[,"re74"]*psid[,"nodegree"],
    psid[,"re74"]*psid[,"black"],
    psid[,"re74"]*psid[,"hispanic"],
    psid[,"re74"]*(psid[,"re75"]==0),
    psid[,"re75"]*psid[,"married"],
    psid[,"re75"]*psid[,"nodegree"],
    psid[,"re75"]*psid[,"black"],
    psid[,"re75"]*psid[,"hispanic"],
    psid[,"re75"]*(psid[,"re74"]==0)
  )
  
  X.dummy.interactions <- cbind(
    psid[,"married"]*psid[,"nodegree"],
    psid[,"married"]*psid[,"black"],
    psid[,"married"]*psid[,"hispanic"],
    psid[,"married"]*(psid[,"re74"]==0),
    psid[,"married"]*(psid[,"re75"]==0),
    psid[,"nodegree"]*psid[,"black"],
    psid[,"nodegree"]*psid[,"hispanic"],
    psid[,"nodegree"]*(psid[,"re74"]==0),
    psid[,"nodegree"]*(psid[,"re75"]==0),
    psid[,"black"]*(psid[,"re74"]==0),
    psid[,"black"]*(psid[,"re75"]==0),
    psid[,"hispanic"]*(psid[,"re74"]==0),
    psid[,"hispanic"]*(psid[,"re75"]==0),
    (psid[,"re74"]==0)*(psid[,"re75"]==0)
  )
  
  X.poly <- poly(cbind(psid[,"age"], psid[,"education"], psid[,"re74"], psid[,"re75"]),degree=5)
  
  X.educ.dummies <- model.matrix(~as.factor(psid[,"education"]) - 1)[,-1]
  
  X <- scale(cbind(X.main,X.cont.interactions,X.dummy.interactions,X.poly),center=FALSE,scale=TRUE)

  p <- dim(X)[2]

   #delta choices, see paper
      delta.Y <- 2
      delta.D <- 3

# Set up the data for group lasso
  #Big matrixes for the grplasso command
    Tnew <- matrix(rep(T),nrow=n,ncol=p)
    Xnew <- matrix(0,nrow=n,ncol=p*Treatments)
    for (t in 0:1) {
      Xnew[,(t*p + 1):((t+1)*p)] <- X*(Tnew==t)
    }

  grouping.index <- rep(1:p,times=Treatments)    

# Initial Group Lasso fit
    #Linear part
	#Iterate to find penalty
	  Y.0.initial <- lm(Y~X.main - 1)$fitted.values
	  Y.0.initial <- lm(Y~1)$fitted.values
	  u.bound <- mean( (Y[T==0] - Y.0.initial[T==0])^4 )^(1/4)
	  x.bound <- max(X)    
	  for (k in 1:5) {
	    lambda.Y <- ( (2*x.bound*u.bound) / (sqrt(min(n.per.treatment)*Treatments)) ) * sqrt(1 + (log(max(p,min(n.per.treatment)))^(3/2 + delta.Y))/sqrt(Treatments) )
	    fit <- grplasso(Xnew, Y, index=grouping.index, model=LinReg(), lambda = lambda.Y, center=FALSE, control=grpl.control(trace=0))
	    S.hat <- sort(union(c(1,3,7,8),which(fit$coefficients[1:p,1]!=0)))
	    Y.0.hat <- X[,S.hat]%*%solve(crossprod(X[T==0,S.hat]))%*%crossprod(X[T==0,S.hat],Y[T==0])
	    u.bound <- mean( (Y[T==0] - Y.0.hat[T==0])^4 )^(1/4)
	  }
	#Estimation
	  fit.grplasso <- grplasso(Xnew, Y, index=grouping.index, model=LinReg(), lambda = lambda.Y, center=FALSE, control=grpl.control(trace=0))
	  
	S.hat.Y <- sort(union(c(1,3,7,8),which((fit.grplasso$coefficients[1:p])!=0)))


    #logit part
	#no need to iterate to find lambda if we use u.bound=1
	  u.bound <- 1	#valid for logit models
	  lambda.T <- ( (x.bound*u.bound*sqrt(Treatments-1)) / (sqrt(min(n.per.treatment))) ) * sqrt(1 + (log(max(p,min(n.per.treatment)))^(3/2 + delta.D))/sqrt(Treatments-1) )

	#Estimation
	  mlogit.glmnet <- glmnet(X, as.factor(T), family=c("multinomial"), alpha=1, lambda=lambda.T/(n*(Treatments-1)), intercept=TRUE, type.multinomial="grouped")
	S.hat.T <- sort(union(c(1,3,7,8),as.numeric(predict(mlogit.glmnet, type="nonzero")$'s0')))

      

# Post selection refitting
      S.hat.Y.more <- S.hat.Y
      S.hat.T.more <- S.hat.T
      
    #Linear part
      Y.0.more.noUnion <- X[,S.hat.Y]%*%(solve(crossprod(X[T==0,S.hat.Y]))%*%crossprod(X[T==0,S.hat.Y],Y[T==0]))

    #logit part
      p.1.more.noUnion <- multinom(T~X[,S.hat.T]-1, trace=FALSE)$fitted.values
    
  #Impose common support
    indexes.to.drop <- which(p.1.more.noUnion < min(p.1.more.noUnion[T==1]) | max(p.1.more.noUnion[T==1]) < p.1.more.noUnion)
    if (length(indexes.to.drop)==0) {indexes.to.drop <- n+1}	#R throws a wobbly if [-indexes.to.drop] is negating an empty set. 
    n.per.treatment.more.noUnion <- as.vector(table(T[-indexes.to.drop]))
    n.trim <- n.per.treatment.more.noUnion[1]+n.per.treatment.more.noUnion[2]

    
  #Treatment Effects on the Treated
    p.1 <- n.per.treatment.more.noUnion[2]/n.trim
    mu01.eif.more.noUnion <- mean((T[-indexes.to.drop]==1)*Y.0.more.noUnion[-indexes.to.drop]) + mean((T[-indexes.to.drop]==0)*p.1.more.noUnion[-indexes.to.drop]*(Y[-indexes.to.drop] - Y.0.more.noUnion[-indexes.to.drop])/(1-p.1.more.noUnion[-indexes.to.drop]))
    mu01.eif.more.noUnion <- mu01.eif.more.noUnion/p.1
    tot.eif.more.noUnion <- mu11.experimental - mu01.eif.more.noUnion
    
    var.tot.eif.more.noUnion <- mean( (T[-indexes.to.drop]==1)*(Y[-indexes.to.drop] - Y.0.more.noUnion[-indexes.to.drop] - tot.eif.more.noUnion)^2 )  + mean( (T[-indexes.to.drop]==0)*((Y[-indexes.to.drop] - Y.0.more.noUnion[-indexes.to.drop])^2)*(p.1.more.noUnion[-indexes.to.drop]^2/(1-p.1.more.noUnion[-indexes.to.drop])^2) )
    var.tot.eif.more.noUnion <- var.tot.eif.more.noUnion/p.1^2


  
    
##################################
## 			 	##
## 	Print Results	 	##
## 			 	##
##################################

    
    
print("experimental")
print(paste(n.per.treatment.experimental[1], " & ", n.per.treatment.experimental[2], "   & &   ",round(tot.experimental), "   & &   ","[", round(tot.experimental - sqrt(var.tot.experimental/n.experimental)*qnorm(0.975)),", ", round(tot.experimental + sqrt(var.tot.experimental/n.experimental)*qnorm(0.975)), "]", sep=""))


print("X and X^2")
n.trim <- n.per.treatment.squared[1]+n.per.treatment.squared[2]
print(paste(n.per.treatment.squared[1], " & ", n.per.treatment.squared[2], "   & &   ",round(tot.eif.squared), "   & &   ","[", round(tot.eif.squared - sqrt(var.tot.eif.squared/n.trim)*qnorm(0.975)),", ", round(tot.eif.squared + sqrt(var.tot.eif.squared/n.trim)*qnorm(0.975)), "]", sep=""))



print("DW02")
n.trim <- n.per.treatment.DW02[1]+n.per.treatment.DW02[2]
print(paste(n.per.treatment.DW02[1], " & ", n.per.treatment.DW02[2], "   & &   ",round(tot.eif.DW02), "   & &   ","[", round(tot.eif.DW02 - sqrt(var.tot.eif.DW02/n.trim)*qnorm(0.975)),", ", round(tot.eif.DW02 + sqrt(var.tot.eif.DW02/n.trim)*qnorm(0.975)), "]", sep=""))



print("Group Lasso")
n.trim <- n.per.treatment.more.noUnion[1]+n.per.treatment.more.noUnion[2]
print(paste(n.per.treatment.more.noUnion[1], " & ", n.per.treatment.more.noUnion[2], "   & &   ",round(tot.eif.more.noUnion), "   & &   ","[", round(tot.eif.more.noUnion - sqrt(var.tot.eif.more.noUnion/n.trim)*qnorm(0.975)),", ", round(tot.eif.more.noUnion + sqrt(var.tot.eif.more.noUnion/n.trim)*qnorm(0.975)), "]", sep=""))



    
 

    
